@@ -261,6 +261,9 @@ class ModelTraining:
         self.save_model_rate = save_model_rate   # [epoch]
         self.save_last_model = save_last_model
         
+        # For model tracking
+        self.last_run_id = None
+        
         # For model steps timing
         self.steps_timing = dict()
         
@@ -463,7 +466,8 @@ class ModelTraining:
             to_be_saved['current_epoch'] = self.current_epoch
             to_be_saved['num_epochs'] = self.num_epochs
             to_be_saved['input_type'] = self.input_type
-            
+            to_be_saved['last_run_id'] = self.last_run_id
+
             params = self.optimizer.param_groups
             lr = [params[i]['lr'] for i in reversed(range(len(params)))]
             to_be_saved['optimizer_lr'] = lr
@@ -496,13 +500,18 @@ class ModelTraining:
         if model_improved:
             # Tracking Experiments 
             if track:
-                self.to_be_logged = {
-                    'params': [('epoch', self.current_epoch), 
-                               ('loss_function', self.loss_function),
-                               ('scheduler', self.scheduler)],
-                               
-                    'metrics': [(key, self.metrics[key][-1]) for key in self.metrics]
-                }
+                if self.last_run_id:
+                    mlflow.delete_run(self.last_run_id)
+                    
+                with mlflow.start_run(run_name=self.model_name) as run:
+                    self.last_run_id = mlflow.active_run().info.run_id
+                    
+                    mlflow.log_param('epoch', self.current_epoch)
+                    mlflow.log_param('loss_function', self.loss_function)
+                    mlflow.log_param('scheduler', self.scheduler)
+                        
+                    for key in self.metrics:
+                        mlflow.log_metric(key, self.metrics[key][-1])
             
             should_be_saved = True
             to_save = {
@@ -542,14 +551,6 @@ class ModelTraining:
         for epoch in range(self.num_epochs):
             with ElapsedTime('One epoch').cpu(with_gpu=False):
                 self.one_epoch(dataset, epoch, best, track)
-                
-        if track:
-            with mlflow.start_run(run_name=self.model_name) as run:
-                for param_name, param_value in self.to_be_logged['params']:
-                    mlflow.log_param(param_name, param_value)
-                    
-                for metric_name, metric_value in self.to_be_logged['metrics']:
-                    mlflow.log_metric(metric_name, metric_value)
                     
     def features_extraction_details(self, n_crops, source_layers, phase, iteration):
         clear_output()
@@ -851,6 +852,7 @@ class ModelTraining:
         model_training.metrics = param_dict['metrics']
         model_training.best_metrics = param_dict['best_metrics']
         model_training.current_epoch = param_dict['current_epoch']
+        model_training.last_run_id = param_dict['last_run_id']
         
         return model_training
       
